package com.joo.ar.gravitytouch.plugin;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;

import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.Handler;
import android.util.Log;

/**
Copyright (c) 2014, Jonathan Besuchet
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer. 
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies, 
either expressed or implied, of the FreeBSD Project.
**/

public class GloveTouchThread extends Thread {
	
    private final BluetoothSocket mmSocket;
    private InputStream mmInStream;
    private OutputStream mmOutStream;
    
    private Handler mGuiHandler;
    
    //private final String GRAVITY_TOUCH_UUID = "3770601a-0b69-44a9-8e94-11801174aab3";
    
    public GloveTouchThread( BluetoothDevice device, Handler guiHandler ) {
        // Use a temporary object that is later assigned to mmSocket,
        // because mmSocket is final
        BluetoothSocket tmp = null;
 
        // Get a BluetoothSocket to connect with the given BluetoothDevice
        try {
            // MY_UUID is the app's UUID string, also used by the server code
        	//UUID uuid = UUID.fromString( GRAVITY_TOUCH_UUID );
            //tmp = device.createInsecureRfcommSocketToServiceRecord( uuid );
            
            Method m = device.getClass().getMethod("createInsecureRfcommSocket", new Class[] { int.class });
            tmp = (BluetoothSocket)m.invoke(device, Integer.valueOf(1));
            
        } catch ( Exception e ) {
        	e.printStackTrace();
        }
        mmSocket = tmp;
        
        mGuiHandler = guiHandler;
    }
 
    public void run() {
        // Cancel discovery because it will slow down the connection
 
        try {
            // Connect the device through the socket. This will block until it succeeds or throws an exception
            Log.i( "Device", "Socket: " + mmSocket.getRemoteDevice().getName() );
            mmSocket.connect();
        } catch (IOException connectException) {
            // Unable to connect; close the socket and get out
            Log.i( "Device", "Failed to connect Gravity touch." );
            try {
                mmSocket.close();
            } catch (IOException closeException) { }
            return;
        }
 
        // Do work to manage the connection (in a separate thread)
        //manageConnectedSocket(mmSocket);
        Log.i( "Device", "Connected to Gravity touch." );
        readWriteLoop( mmSocket );
    }
    
    
    private void readWriteLoop( BluetoothSocket socket ) {
        InputStream tmpIn = null;
        OutputStream tmpOut = null;
 
        // Get the input and output streams, using temp objects because
        // member streams are final
        try {
            tmpIn = socket.getInputStream();
            tmpOut = socket.getOutputStream();
        } catch (IOException e) { }
 
        mmInStream = tmpIn;
        mmOutStream = tmpOut;
	
	    // Keep listening to the InputStream until an exception occurs
        String str = new String();
	    while (true) {
	        try {
	            // Read from the InputStream
	        	int b = mmInStream.read();
	        	
	        	if ( b == -1 || (char)b == '\n' ) {
	        		// Send the obtained data to the UI activity
	        		Log.i( "Data", str );
	        		if ( str.charAt( 0 ) == 'y' ) { // receiving yaw-pitch-roll data. Format: "yFloat,Float,Float"
	        			//String ypr = str.substring( 1 );
		        		mGuiHandler.obtainMessage( 101, str).sendToTarget();
		        		//Log.i( "Data", ypr );
	        		}
	        		else if ( str.charAt( 0 ) == 'q' ) { // receiving quaternion data. Format: "qFloat,Float,Float,Float"
	        			//String q = str.substring( 1 );
		        		mGuiHandler.obtainMessage( 102, str).sendToTarget();
		        		//Log.i( "Data", ypr );
	        		}
	        		else if ( str.charAt( 0 ) == 'f' ) { // receiving finger touch event. Format: "fpInt" for press, "frInt" for release.
		        		//int state = str.charAt( 1 ) == 'p' ? 1 : 0; // p = pressed; r = released
		        		//int fingerNo = str.charAt( 2 ) - 48; // '0' = character number 48.
		        		mGuiHandler.obtainMessage( 100, str).sendToTarget();
		        		//Log.i( "Data", "Finger "+fingerNo+" "+(state==0?"pressed":"released") );
	        		}
	        		
	        		str = new String();
	        	}
	        	else {
	        		str += (char)b;
	        	}
	        } catch (IOException e) {
	            break;
	        }
	    }
	}
	
	/* Call this from the main activity to send data to the remote device */
	public void write(byte[] bytes) {
	    try {
	        mmOutStream.write(bytes);
	    } catch (IOException e) { }
	}
	
	/* Call this from the main activity to shutdown the connection */
	public void cancel() {
	    try {
	        mmSocket.close();
	    } catch (IOException e) { }
	}
	
}