package com.joo.ar.gravitytouch.plugin;

import java.util.Set;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.WindowManager;
import android.widget.Toast;

import com.joo.ar.gravitytouch.dialogs.BluetoothNoSupportedDialod;

/**
Copyright (c) 2014, Jonathan Besuchet
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer. 
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies, 
either expressed or implied, of the FreeBSD Project.
**/

public class MainActivity extends Activity implements Handler.Callback {
	
	public final int REQUEST_ENABLE_BT = 1;
	public final String GRAVITY_TOUCH_ID = "Gravity touch";
	
	private BluetoothAdapter mBluetoothAdapter;
	private BluetoothDevice mBtDevice;
	private BroadcastReceiver mReceiver = null;
	static private GloveTouchThread mGloveTouchThread = null;
	private Handler mGloveHandler;

	@Override
	protected void onCreate ( Bundle savedInstanceState ) {
		super.onCreate( savedInstanceState );
		
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        
        mGloveHandler = new Handler( this );
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		
		// Get Bluetooth adapter and check if device supports Bluetooth.
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		
		if ( mBluetoothAdapter == null ) { // Device does not support Bluetooth
			BluetoothNoSupportedDialod btnsd = new BluetoothNoSupportedDialod();
			btnsd.show( this.getFragmentManager(), btnsd.TAG );
			return;
		}
		
		if ( !mBluetoothAdapter.isEnabled() ) { // ask the user to enable Bluetooth module.
		    Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
		    startActivityForResult( enableBtIntent, REQUEST_ENABLE_BT );
		}
		else {
			startBluetoothConnection ( mBluetoothAdapter );
		}
	}

	@Override
	public void onPause () {
		super.onPause();
		
		if ( mReceiver != null ) {
			unregisterReceiver( mReceiver );
		}
		
		if ( mGloveTouchThread != null ) {
			mGloveTouchThread.cancel();
		}
	}
	
	@Override
	public boolean onCreateOptionsMenu ( Menu menu ) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
	
	@Override
	public void onActivityResult ( int requestCode, int resultCode, Intent data ) {
		
		if ( requestCode == REQUEST_ENABLE_BT ) {
			if ( resultCode == RESULT_OK ) {
				Toast.makeText( this, R.string.bluetooth_started, Toast.LENGTH_LONG ).show();
				startBluetoothConnection ( mBluetoothAdapter );
			}
			else {
				Toast.makeText( this, R.string.bluetooth_not_started, Toast.LENGTH_LONG ).show();
			}
		}
		
	}
	
	private void startBluetoothConnection ( BluetoothAdapter bluetoothAdapter ) {
		
		mBtDevice = getGravityTouchDevice ( bluetoothAdapter );
		if ( mBtDevice != null ) {
			Log.i( "Device", "Gravity touch device fund." );
			startConnectionThread();
		}
		else {
			Log.i( "Device", "Gravity touch device not already paired.\nStart discovering devices." );
			
			// Create a BroadcastReceiver for ACTION_FOUND
			mReceiver = new BroadcastReceiver() {
				public void onReceive(Context context, Intent intent) {
			        String action = intent.getAction();
			        // When discovery finds a device
			        if (BluetoothDevice.ACTION_FOUND.equals(action)) {
			            // Get the BluetoothDevice object from the Intent
			            BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
			            if ( device.getName().equals( GRAVITY_TOUCH_ID ) ) {
			    			Log.i( "Device", "Gravity touch device fund." );
				    		mBtDevice = device;
				    		startConnectionThread();
				    		mBluetoothAdapter.cancelDiscovery();
				    	}
			        }
			    }
			};
			// Register the BroadcastReceiver
			IntentFilter filter = new IntentFilter( BluetoothDevice.ACTION_FOUND );
			registerReceiver( mReceiver, filter ); // Don't forget to unregister during onDestroy
			
			bluetoothAdapter.startDiscovery();
		}
		
	}
	
	private void startConnectionThread () {
		
		mGloveTouchThread = new GloveTouchThread( mBtDevice, mGloveHandler );
		mGloveTouchThread.start();
		
	}
	
	private BluetoothDevice getGravityTouchDevice ( BluetoothAdapter bluetoothAdapter ) {
		
		Set<BluetoothDevice> pairedDevices = bluetoothAdapter.getBondedDevices();
		
		if ( pairedDevices.size() > 0 ) { // If there are paired devices
		    for ( BluetoothDevice device : pairedDevices ) { // Loop through paired devices
		    	Log.i( "Paired devices", device.getName() );
		    	if ( device.getName().equals( GRAVITY_TOUCH_ID ) ) {
		    		return device;
		    	}
		    }
		}
		
		return null;
	}
	
	
	static public void EnableGyro () {
		String cmd = "q";
		mGloveTouchThread.write( cmd.getBytes() );
	}
	
	static public void DisableGyro () {
		String cmd = "e";
		mGloveTouchThread.write( cmd.getBytes() );
	}
	
	public void sendYPR ( View v ) {
		Log.i( "Cmd", "sendYPR" );
		String cmd = "y";
		mGloveTouchThread.write( cmd.getBytes() );
	}
	
	public void sendQuat ( View v ) {
		Log.i( "Cmd", "sendQuaternion" );
		String cmd = "q";
		mGloveTouchThread.write( cmd.getBytes() );
	}
	
	public void setGyroData ( float yaw, float roll, float pitch ) {
		/* TODO implements send message to notify Unity.
		 * 
		 */
	}
	
	@Override
	public boolean handleMessage ( Message gravityTouchMsg ) {
		
		if ( gravityTouchMsg.what == 100 ) {
			String fingerStr = gravityTouchMsg.obj.toString();
			Log.i( "Finger touch event", fingerStr );
		}
		else if ( gravityTouchMsg.what == 102 ) {
			String quatStr = gravityTouchMsg.obj.toString();
			Log.i( "Quaternion of wrist orientation", quatStr );
		}
		return false;
	}

}
